# coding=utf-8
import os
import sys
import shutil
import xml.etree.ElementTree as ET

from zip_utils import zip_dirs
import Tucker
from modeldatahandler import ModelDataHandler
from modelservicecontext import ERequestResponseDataFlag
from modelservicecontext import ERequestResponseDataMIME
from modelservicecontext import ModelServiceContext


# Begin
if len(sys.argv) < 4:
    exit()

ms = ModelServiceContext()
ms.onInitialize(sys.argv[1], sys.argv[2], sys.argv[3])
mdh = ModelDataHandler(ms)

# 创建instance文件夹
instance_dir = ms.getModelInstanceDirectory()
if not os.path.exists(instance_dir):
    os.makedirs(instance_dir)

# Enter State
ms.onEnterState('LoadVariables')

train_file_path = None
# rank_udx_file = None
x = y = z = None
data_dir = os.path.join(instance_dir, "data")

ms.onFireEvent('Train')
ms.onRequestData()
if ms.getRequestDataFlag() == ERequestResponseDataFlag.ERDF_OK:
    if ms.getRequestDataMIME() == ERequestResponseDataMIME.ERDM_RAW_FILE:
        train_file_path = ms.getRequestDataBody()
        if not os.path.splitext(train_file_path)[1] == ".csv":
            print("This model do not support this format input data!")
            ms.onFinalize()
    else:
        print("This model do not support this format input data!")
        ms.onFinalize()
else:
    print("Input data failed!")
    ms.onFinalize()

ms.onFireEvent('Rank')
ms.onRequestData()
if ms.getRequestDataFlag() == ERequestResponseDataFlag.ERDF_OK:
    if ms.getRequestDataMIME() == ERequestResponseDataMIME.ERDM_XML_FILE:
        # rank_udx_file = ms.getRequestDataBody()
        rank_tree = ET.ElementTree()
        rank_tree.parse(ms.getRequestDataBody())

        value_node_list = rank_tree.findall("XDO")
        x = int(value_node_list[0].attrib['value'])
        y = int(value_node_list[1].attrib['value'])
        z = int(value_node_list[2].attrib['value'])

    else:
        print("This model do not support this format input data!")
        ms.onFinalize()
else:
    print("Input data failed!")
    ms.onFinalize()

ms.onLeaveState()

ms.onEnterState('Run_And_Output')
print("Model is running!")
shutil.copytree(os.path.join(os.getcwd(), 'model/bj'), os.path.join(instance_dir, 'bj'))

# 切换工作目录
os.chdir(os.path.dirname(instance_dir))
Tucker.Tucker(train_file_path, x, y, z)

# 压缩图片输出
ms.onFireEvent('picture_output')
picture_out_dir = os.path.join(instance_dir, "pic")
picture_zip = os.path.join(instance_dir, "picture.zip")

if not os.path.exists(picture_out_dir):
    print("Model error!")
    ms.onFinalize()

zip_dirs([picture_out_dir], picture_zip)
ms.setResponseDataFlag(ERequestResponseDataFlag.ERDF_OK)
ms.setResponseDataMIME(ERequestResponseDataMIME.ERDM_ZIP_FILE)
ms.setResponseDataBody(picture_zip)
ms.onResponseData()

# 压缩shp输出
ms.onFireEvent('shapefile_output')
shp_out_dir = os.path.join(instance_dir, "outshp")
out_shp_zip = os.path.join(instance_dir, "out_shp.zip")

if not os.path.exists(shp_out_dir):
    print("Model error!")
    ms.onFinalize()

zip_dirs([shp_out_dir], out_shp_zip)
ms.setResponseDataFlag(ERequestResponseDataFlag.ERDF_OK)
ms.setResponseDataMIME(ERequestResponseDataMIME.ERDM_ZIP_FILE)
ms.setResponseDataBody(out_shp_zip)
ms.onResponseData()

ms.onLeaveState()
ms.onFinalize()
